from softcolor.morphology import MorphologyInCIELab, soften_structuring_element
from skimage.morphology import disk
from skimage import img_as_float
import numpy


def inpaint_from_mask(image, mask):
    morphology = MorphologyInCIELab()
    se = disk(1).astype('float64')

    se[se == 0] = numpy.nan

    image_with_nans = generate_image_with_nans(image, mask)
    image_inpainted, img_inpainted_steps = morphology.inpaint_with_steps(image_with_nans,
                                                                         structuring_element=se,
                                                                         max_iterations=100)

    return image_inpainted


def generate_image_with_nans(image, mask):
    image_size = image.shape
    image_with_nans = img_as_float(image.copy())
    for i in range(image_size[0]):
        for j in range(image_size[1]):
            if mask[i, j]:
                image_with_nans[i, j, 0] = numpy.nan
                image_with_nans[i, j, 1] = numpy.nan
                image_with_nans[i, j, 2] = numpy.nan

    return image_with_nans



from staplesremoval.staples_detector import *
from staplesremoval.inpainting import *
from staplesremoval.remove_staples import *
from staplesremoval.trapezoidal_fuzzy_color_segmentation import trapezoidal_fuzzy_color_segmentation
from staplesremoval.triangular_fuzzy_color_segmentation import triangular_fuzzy_color_segmentation


import numpy
from skimage.color import rgb2hsv
from staplesremoval import remove_staples
from staplesremoval import triangular_fuzzy_color_segmentation
from staplesremoval.triangular_fuzzy_color_segmentation import get_number_red_pixels
from skimage import io


def detect_infection(image):
    regions = generate_wound_regions(image, 5, 5)
    clean_regions = remove_staples_from_regions(regions)
    segmented_regions = segment_inpainted_regions(clean_regions)

    if len(segmented_regions) == 0:
        return 0
    else:
        return get_red_proportion(segmented_regions)


def get_red_proportion(regions):
    total_pixels = 0
    total_red_pixels = 0
    for image in regions:
        total, total_red = get_number_red_pixels(image)
        total_pixels += total
        total_red_pixels += total_red

    return total_red_pixels/total_pixels


def segment_inpainted_regions(regions):
    segmented_regions = [triangular_fuzzy_color_segmentation(image) for image in regions]
    return segmented_regions


def remove_staples_from_regions(regions):
    cleaned_regions = [remove_staples(image) for image in regions]
    return cleaned_regions


def generate_wound_regions(image, N_vertical, N_horizontal):
    rgb_image = image.copy()
    height = rgb_image.shape[0]
    width = rgb_image.shape[1]

    vertical_grid = generate_grid(width, N_vertical)
    horizontal_grid = generate_grid(height, N_horizontal)

    wound_images = []

    for i in range(0, len(vertical_grid) - 1):
        for j in range(0, len(horizontal_grid) - 1):
            subimage = rgb_image[horizontal_grid[j]:horizontal_grid[j + 1], vertical_grid[i]:vertical_grid[i + 1], :]
            if image_contains_wound(subimage):
                wound_images.append(subimage)

    return wound_images


def generate_grid(size, N):
    step = int(size / N)
    sequence = [(step * i) for i in range(0, 5)]
    sequence.append(size)

    return sequence


def image_contains_wound(image):
    hsv_image = rgb2hsv(image)
    h_range = numpy.max(hsv_image[:, :, 0]) - numpy.min(hsv_image[:, :, 0])
    v_range = numpy.max(hsv_image[:, :, 2]) - numpy.min(hsv_image[:, :, 2])

    if v_range >= 0.8886531 - 0.1108036 * h_range:
        return True
    else:
        return False



from skimage import color


def get_number_red_pixels(image):
    total_pixels = image.shape[0]*image.shape[1]

    red_pixels = 0;
    for i in range(image.shape[0]):
        for j in range(image.shape[1]):
            if image[i, j, 0] == 255 & image[i, j, 1] == 0 & image[i, j, 2] == 0:
                red_pixels += 1

    return total_pixels, red_pixels


def trapezoidal_fuzzy_color_segmentation(image):
    rgb_image = image.copy()
    hsv_image = color.rgb2hsv(rgb_image)

    h_channel = 360 * hsv_image[:, :, 0]

    for i in range(h_channel.shape[0]):
        for j in range(h_channel.shape[1]):

            p_red = fuzzy_segmentation_red_trapezoidal(h_channel[i, j])
            p_orange = fuzzy_segmentation_orange_trapezoidal(h_channel[i, j])
            p_yellow = fuzzy_segmentation_yellow_trapezoidal(h_channel[i, j])
            p_green = fuzzy_segmentation_green_trapezoidal(h_channel[i, j])
            p_cyan = fuzzy_segmentation_cyan_trapezoidal(h_channel[i, j])
            p_blue = fuzzy_segmentation_blue_trapezoidal(h_channel[i, j])
            p_purple = fuzzy_segmentation_purple_trapezoidal(h_channel[i, j])

            m = max([p_red, p_orange, p_yellow, p_green, p_cyan, p_blue, p_purple])

            if m == p_red:
                rgb_image[i, j, 0] = 255
                rgb_image[i, j, 1] = 0
                rgb_image[i, j, 2] = 0
            elif m == p_orange:
                rgb_image[i, j, 0] = 255
                rgb_image[i, j, 1] = 165
                rgb_image[i, j, 2] = 0
            elif m == p_yellow:
                rgb_image[i, j, 0] = 255
                rgb_image[i, j, 1] = 255
                rgb_image[i, j, 2] = 0
            elif m == p_green:
                rgb_image[i, j, 0] = 0
                rgb_image[i, j, 1] = 128
                rgb_image[i, j, 2] = 0
            elif m == p_cyan:
                rgb_image[i, j, 0] = 0
                rgb_image[i, j, 1] = 255
                rgb_image[i, j, 2] = 255
            elif m == p_blue:
                rgb_image[i, j, 0] = 0
                rgb_image[i, j, 1] = 0
                rgb_image[i, j, 2] = 255
            elif m == p_purple:
                rgb_image[i, j, 0] = 128
                rgb_image[i, j, 1] = 0
                rgb_image[i, j, 2] = 128

    return rgb_image


def fuzzy_segmentation_red_trapezoidal(h):
    if (0 <= h <= 10) or (330 < h <= 360):
        return (1)
    elif 10 < h <= 20:
        return -0.1 * h + 2
    elif 20 < h <= 300:
        return 0
    elif 300 < h <= 330:
        return (1 / 30) * h - 10


def fuzzy_segmentation_orange_trapezoidal(h):
    if 0 <= h <= 10 or 55 < h <= 360:
        return 0
    elif 10 < h <= 20:
        return 0.1 * h - 1
    elif 20 < h <= 40:
        return 1
    elif 40 < h <= 55:
        return (-1 / 15) * h + 11 / 3


def fuzzy_segmentation_yellow_trapezoidal(h):
    if 0 <= h <= 40 or 80 < h <= 360:
        return 0
    elif 40 < h <= 55:
        return (1 / 15) * h - 8 / 3
    elif 55 < h <= 65:
        return 1
    elif 65 < h <= 80:
        return (-1 / 15) * h + 11 / 3


def fuzzy_segmentation_green_trapezoidal(h):
    if 0 <= h <= 65 or 170 < h <= 360:
        return 0
    elif 65 < h <= 80:
        return (1 / 15) * h - 8 / 3
    elif 80 < h <= 140:
        return 1
    elif 140 < h <= 170:
        return (-1 / 30) * h + 17 / 3


def fuzzy_segmentation_cyan_trapezoidal(h):
    if 0 <= h <= 140 or 210 < h <= 360:
        return 0
    elif 140 < h <= 170:
        return (1 / 30) * h - 14 / 3
    elif 170 < h <= 200:
        return 1
    elif 200 < h <= 210:
        return -0.1 * h + 21


def fuzzy_segmentation_blue_trapezoidal(h):
    if 0 <= h <= 200 or 270 < h <= 360:
        return 0
    elif 200 < h <= 210:
        return 0.1 * h - 20
    elif 210 < h <= 250:
        return 1
    elif 250 < h <= 270:
        return (-1 / 20) * h + 27 / 2


def fuzzy_segmentation_purple_trapezoidal(h):
    if 0 <= h <= 250 or 330 < h <= 360:
        return 0
    elif 250 < h <= 270:
        return (1 / 20) * h - 5 / 4
    elif 270 < h <= 300:
        return 1
    elif 300 < h <= 330:
        return (-1 / 30) * h + 11



import skimage
from skimage.color import rgb2hsv
from skimage import morphology
import numpy
import scipy


def generate_mask(image):
    hsv_image = convert_image_to_HSV(image)
    V = get_V_channel(hsv_image)

    GY = calculate_sobel_vertical_gradient(V)
    BY = binarize_image(GY, 60 / 255)

    se = numpy.array([[1, 1, 1], [1, 1, 1], [1, 1, 1]])
    C = morphological_closing(BY, se)

    holes = fill_small_holes(C)
    filtered = remove_small_objects(holes, 50)

    dilated_mask = dilate_mask(dilate_mask(dilate_mask(dilate_mask(filtered, se), se), se),se)
    colormask = get_drawn_mask(image, dilated_mask)

    return colormask, dilated_mask


def convert_image_to_HSV(image):
    return (rgb2hsv(image))


def get_V_channel(hsvImage):
    return (hsvImage[:, :, 2])


def calculate_sobel_vertical_gradient(image):
    ky = numpy.array([[1, 2, 1],
                      [0, 0, 0],
                      [-1, -2, -1]])
    return (scipy.ndimage.convolve(image, ky))


def binarize_image(image, threshold):
    binary = image.copy()
    binary[binary > threshold] = 1
    binary[binary <= threshold] = 0

    return binary


def morphological_closing(image, structuring_element):
    return (skimage.morphology.binary_closing(image,
                                              structuring_element))


def fill_small_holes(image):
    return (scipy.ndimage.morphology.binary_fill_holes(image))


def remove_small_objects(image, area):
    return (skimage.morphology.remove_small_objects(image, area))


def dilate_mask(mask, structuring_element):
    return (skimage.morphology.binary_dilation(mask,
                                               structuring_element))

def get_drawn_mask(image,mask):

    colormask = image.copy()
    for i in range(mask.shape[0]):
        for j in range(mask.shape[1]):
            if mask[i, j]:
                colormask[i, j, 0] = 70/255
                colormask[i, j, 1] = 253/255
                colormask[i, j, 2] = 52/255

    return colormask



from skimage import color


def get_number_red_pixels(image):
    total_pixels = image.shape[0]*image.shape[1]

    red_pixels = 0;
    for i in range(image.shape[0]):
        for j in range(image.shape[1]):
            if image[i, j, 0] == 255 and image[i, j, 1] == 0 and image[i, j, 2] == 0:
                red_pixels += 1

    return total_pixels, red_pixels


def triangular_fuzzy_color_segmentation(image):
    rgb_image = image.copy()
    hsv_image = color.rgb2hsv(rgb_image)

    h_channel = 360 * hsv_image[:, :, 0]

    for i in range(h_channel.shape[0]):
        for j in range(h_channel.shape[1]):

            p_red = fuzzy_segmentation_red_triangular(h_channel[i, j])
            p_darkorange = fuzzy_segmentation_darkorange_triangular(h_channel[i, j])
            p_lightorange = fuzzy_segmentation_lightorange_triangular(h_channel[i, j])
            p_yellow = fuzzy_segmentation_yellow_triangular(h_channel[i, j])
            p_lightgreen = fuzzy_segmentation_lightgreen_triangular(h_channel[i, j])
            p_darkgreen = fuzzy_segmentation_darkgreen_triangular(h_channel[i, j])
            p_aqua = fuzzy_segmentation_aqua_triangular(h_channel[i, j])
            p_blue = fuzzy_segmentation_blue_triangular(h_channel[i, j])
            p_darkpurple = fuzzy_segmentation_darkpurple_triangular(h_channel[i, j])
            p_lightpurple = fuzzy_segmentation_lightpurple_triangular(h_channel[i, j])

            m = max([p_red, p_darkorange, p_lightorange, p_yellow, p_lightgreen, p_darkgreen,
                     p_aqua, p_blue, p_darkpurple, p_lightpurple])

            if m == p_red:
                rgb_image[i, j, 0] = 255
                rgb_image[i, j, 1] = 0
                rgb_image[i, j, 2] = 0
            elif m == p_darkorange:
                rgb_image[i, j, 0] = 255
                rgb_image[i, j, 1] = 140
                rgb_image[i, j, 2] = 0
            elif m == p_lightorange:
                rgb_image[i, j, 0] = 255
                rgb_image[i, j, 1] = 165
                rgb_image[i, j, 2] = 0
            elif m == p_yellow:
                rgb_image[i, j, 0] = 255
                rgb_image[i, j, 1] = 255
                rgb_image[i, j, 2] = 0
            elif m == p_lightgreen:
                rgb_image[i, j, 0] = 144
                rgb_image[i, j, 1] = 238
                rgb_image[i, j, 2] = 144
            elif m == p_darkgreen:
                rgb_image[i, j, 0] = 0
                rgb_image[i, j, 1] = 100
                rgb_image[i, j, 2] = 0
            elif m == p_aqua:
                rgb_image[i, j, 0] = 0
                rgb_image[i, j, 1] = 255
                rgb_image[i, j, 2] = 255
            elif m == p_blue:
                rgb_image[i, j, 0] = 0
                rgb_image[i, j, 1] = 0
                rgb_image[i, j, 2] = 255
            elif m == p_darkpurple:
                rgb_image[i, j, 0] = 128
                rgb_image[i, j, 1] = 0
                rgb_image[i, j, 2] = 128
            elif m == p_lightpurple:
                rgb_image[i, j, 0] = 255
                rgb_image[i, j, 1] = 0
                rgb_image[i, j, 2] = 255

    return rgb_image


def fuzzy_segmentation_red_triangular(h):
    if 0 <= h <= 30:
        return 1 - h / 30
    elif 330 <= h <= 360:
        return -11 + h / 30
    else:
        return 0


def fuzzy_segmentation_darkorange_triangular(h):
    if 0 <= h <= 30:
        return h / 30
    elif 30 <= h <= 45:
        return 3 - h / 15
    else:
        return 0


def fuzzy_segmentation_lightorange_triangular(h):
    if 30 <= h <= 45:
        return -2 + h / 15
    elif 45 <= h <= 60:
        return 4 - h / 15
    else:
        return 0


def fuzzy_segmentation_yellow_triangular(h):
    if 45 <= h <= 60:
        return -3 + h / 15
    elif 60 <= h <= 90:
        return 3 - h / 30
    else:
        return 0


def fuzzy_segmentation_lightgreen_triangular(h):
    if 60 <= h <= 75:
        return -4 + h / 15
    elif 75 <= h <= 120:
        return 8 / 3 - h / 45
    else:
        return 0


def fuzzy_segmentation_darkgreen_triangular(h):
    if 90 <= h <= 120:
        return -3 + h / 30
    elif 120 <= h <= 180:
        return 3 - h / 60
    else:
        return 0


def fuzzy_segmentation_aqua_triangular(h):
    if 120 <= h <= 180:
        return -2 + h / 60
    elif 180 <= h <= 240:
        return 4 - h / 60
    else:
        return 0


def fuzzy_segmentation_blue_triangular(h):
    if 180 <= h <= 240:
        return -3 + h / 60
    elif 240 <= h <= 300:
        return 5 - h / 60
    else:
        return 0


def fuzzy_segmentation_darkpurple_triangular(h):
    if 240 <= h <= 300:
        return -4 + h / 60
    elif 300 <= h <= 330:
        return 11 - h / 30
    else:
        return 0


def fuzzy_segmentation_lightpurple_triangular(h):
    if 300 <= h <= 330:
        return -10 + h / 30
    elif 330 <= h <= 360:
        return 12 - h / 30;
    else:
        return 0



import skimage
import os as _os
import os.path as osp
from skimage import *

data_dir = osp.abspath(osp.dirname(__file__))


def load_infection_example_image():
    return skimage.io.imread(_os.path.join(data_dir, 'images/image_infection.png'))


def load_noinfection_example_image():
    return skimage.io.imread(_os.path.join(data_dir, 'images/image_noinfection.png'))



from setuptools import setup

setup(
    name='staplesremoval',
    version='1.4',
    packages=['staplesremoval'],
    url='https://github.com/mmunar97',
    license='mit',
    author='marcmunar',
    author_email='marc.munar1@estudiant.uib.es',
    description='Algorithm for staple detection and mask generation',
    include_package_data=True,
    install_requires = [
        'matplotlib',
        'scipy',
        'scikit-image',
        'softcolor'
    ]
)



from staplesremoval import staples_detector
from staplesremoval import  inpainting

def remove_staples(image):
    colormask, mask = staples_detector.generate_mask(image)
    inpainted = inpainting.inpaint_from_mask(image, mask)
    return(inpainted)